@@ -39,7 +39,7 @@ def dispatcher(_command,
         method = _command
         if _subc is not None:
             method += '_' + _subc.replace('.', '_')
-        getattr(common.cli, method)(**kwargs)
+        getattr(common.cli, method.replace('-', '_'))(**kwargs)
         current_command.exit_code = 0
     except RpcException as e:
         current_command.exit_code = 3
@@ -793,6 +793,19 @@ def append_venv_cli(root_sp):
                    action='store_true',
                    help='automatically rebuild venv')
 
+    mirror_dir = f'{common.dir_eva}/mirror'
+    p = sp.add_parser('mirror-update', help='Create/update PyPi mirror')
+    p.add_argument('--dest',
+                   metavar='DIR',
+                   help=f'Mirror directory (default: {mirror_dir})',
+                   default=mirror_dir)
+
+    p = sp.add_parser('mirror-set', help='Set PyPi mirror URL')
+    p.add_argument('mirror_url',
+                   metavar='URL',
+                   help=('EVA ICS v4 mirror url as http://<ip/host>:port'
+                         ' or "default" to restore the default settings'))
+
 
 def append_cloud_cli(root_sp):
     ap = root_sp.add_parser('cloud', help='cloud manager commands')
@@ -805,7 +818,11 @@ def append_cloud_cli(root_sp):
                    '--config-var',
                    metavar='VAR',
                    action='append',
-                   help='config var name=value').completer = ComplYamlFile()
+                   help='config var name=value')
+    p.add_argument('--config',
+                   metavar='FILE',
+                   help='load configuration variables from YAML file'
+                  ).completer = ComplYamlFile()
     p.add_argument('--test',
                    action='store_true',
                    help=f'test deployment config and exit')
@@ -817,11 +834,32 @@ def append_cloud_cli(root_sp):
                    '--config-var',
                    metavar='VAR',
                    action='append',
-                   help='config var name=value').completer = ComplYamlFile()
+                   help='config var name=value')
+    p.add_argument('--config',
+                   metavar='FILE',
+                   help='load configuration variables from YAML file'
+                  ).completer = ComplYamlFile()
     p.add_argument('--test',
                    action='store_true',
                    help=f'test deployment config and exit')
 
+    p = sp.add_parser('update', help='update cloud nodes')
+    p.add_argument('nodes', metavar='NODE', nargs='*').completer = ComplNode()
+    p.add_argument('--check-timeout',
+                   help='Max node update duration (default: 120 sec)',
+                   default=120,
+                   metavar='SEC',
+                   type=float)
+    p.add_argument('--all', help='update all nodes', action='store_true')
+    p.add_argument('-i',
+                   '--info-only',
+                   help='display the update plan and exit',
+                   action='store_true')
+    p.add_argument('--YES',
+                   dest='yes',
+                   help='update without any confirmations',
+                   action='store_true')
+
 
 def append_system_cli(root_sp):
     ap = root_sp.add_parser('system', help='system commands')
@@ -832,6 +870,38 @@ def append_system_cli(root_sp):
     p = sp.add_parser('poweroff', help='power off the system')
 
 
+def append_mirror_cli(root_sp):
+    ap = root_sp.add_parser('mirror', help='mirror commands')
+    sp = ap.add_subparsers(dest='_subc', help='sub command')
+
+    mirror_dir = f'{common.dir_eva}/mirror'
+    p = sp.add_parser('update', help='update the mirror')
+    p.add_argument('--dest',
+                   metavar='DIR',
+                   help=f'Mirror directory (default: {mirror_dir})',
+                   default=mirror_dir)
+    p.add_argument('-o',
+                   '--current-arch-only',
+                   help='download files for the current CPU architecture only',
+                   action='store_true')
+    p.add_argument('--force',
+                   help='force download existing files',
+                   action='store_true')
+    p.add_argument('-u',
+                   '--repository-url',
+                   metavar='URL',
+                   help='repository url')
+
+    p = sp.add_parser('info', help='get mirror info')
+
+    p = sp.add_parser(
+        'set', help='Set mirror URL (do not run this on the mirror host node)')
+    p.add_argument('mirror_url',
+                   metavar='URL',
+                   help=('EVA ICS v4 mirror url as http://<ip/host>:port'
+                         ' or "default" to restore the default settings'))
+
+
 def sys_cmd(cmd):
     if cmd == 'top':
         import distutils.spawn
@@ -879,6 +949,7 @@ def init_ap():
         'key': [],
         'user': [],
         'system': [],
+        'mirror': []
     }
 
     ap.add_argument('-D',
@@ -908,6 +979,7 @@ def init_ap():
     append_venv_cli(sp)
     append_cloud_cli(sp)
     append_system_cli(sp)
+    append_mirror_cli(sp)
 
     sp.add_parser('save', help='save scheduled states (if instant-save is off)')
     append_svc_cli(sp)
@@ -926,29 +998,25 @@ def init_ap():
         'for keys: connect directly to the registry db when the node is offline'
     )
 
-    try:
-        import requests
-        from . import REPOSITORY_URL
-        p = sp.add_parser('update', help='update the node')
-        p.add_argument('-u',
-                       '--repository-url',
-                       metavar='URL',
-                       help='repository url',
-                       default=REPOSITORY_URL)
-        p.add_argument('--YES',
-                       dest='yes',
-                       action='store_true',
-                       help='update without a confirmation')
-        p.add_argument(
-            '-i',
-            '--info-only',
-            action='store_true',
-            help='do not perform the actual update, get the info only')
-        p.add_argument('--test',
-                       action='store_true',
-                       help='install a build marked as test')
-    except ModuleNotFoundError:
-        pass
+    p = sp.add_parser('update', help='update the node')
+    p.add_argument('--download-timeout',
+                   type=float,
+                   help='update file download timeout (default: timeout*30)')
+    p.add_argument('-u',
+                   '--repository-url',
+                   metavar='URL',
+                   help='repository url')
+    p.add_argument('--YES',
+                   dest='yes',
+                   action='store_true',
+                   help='update without a confirmation')
+    p.add_argument('-i',
+                   '--info-only',
+                   action='store_true',
+                   help='get update info only')
+    p.add_argument('--test',
+                   action='store_true',
+                   help='install a build marked as test')
 
     sp.add_parser('version', help='core version')
 
