@@ -1,7 +1,6 @@
 import os
 import sys
 import time
-import socket
 from functools import partial
 
 import elbus
@@ -12,9 +11,12 @@ from rapidtables import format_table, FORMAT_GENERATOR, FORMAT_GENERATOR_COLS
 from .tools import print_result, ok, edit_config, read_file, write_file
 from .tools import print_action_result, format_value, prepare_time, err, warn
 from .tools import get_node_svc_info, edit_file, get_term_size, exec_cmd, xc
+from .tools import get_my_ip
 from .client import call_rpc, DEFAULT_REPL_SERVICE
 from .sharedobj import common, current_command
 
+from . import DEFAULT_REPOSITORY_URL
+
 
 def eva_control_c(c, pfx=''):
     cmd = f'{pfx}{common.dir_eva}/sbin/eva-control {c}'
@@ -26,17 +28,24 @@ class CLI:
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         for vc in [
-                'build', 'list', 'edit', 'config', 'add', 'remove', 'update'
+                'build', 'list', 'edit', 'config', 'add', 'remove', 'update',
+                'mirror-update', 'mirror-set'
         ]:
-            setattr(self, f'venv_{vc}', partial(self.venv_cmd, vc))
+            setattr(self, f'venv_{vc.replace("-", "_")}',
+                    partial(self.venv_cmd, vc))
 
     def venv_cmd(self, vc, **kwargs):
-        args = vc
+        args = '--verbose ' if current_command.debug else ''
+        args += vc
         args += ' ' + ' '.join(kwargs.get('modules', []))
+        if 'mirror_url' in kwargs:
+            args += f' "{kwargs["mirror_url"]}"'
         if kwargs.get('rebuild'):
             args += ' --rebuild'
         if kwargs.get('from_scratch'):
             args += ' --from-scratch'
+        if 'dest' in kwargs:
+            args += f' --dest "{kwargs["dest"]}"'
         exec_cmd('venvmgr', args, search_in='sbin', search_system=False)
 
     def test(self):
@@ -79,164 +88,66 @@ class CLI:
     def server_status(self):
         eva_control_c('status')
 
-    def cloud_deploy(self, file, test, config_var):
+    def cloud_deploy(self, file, config_var, config, test):
         args = (f'cloud deploy --timeout {current_command.timeout}'
                 f' --connection-path "{common.elbus_path}" "{file}"')
         if config_var:
             for c in config_var:
                 args += f' --config-var "{c}"'
+        if config:
+            args += f' --config "{config}"'
         if test:
             args += ' --test'
         exec_cmd('eva-cloud-manager', args)
 
-    def cloud_undeploy(self, file, test, config_var):
+    def cloud_undeploy(self, file, config_var, config, test):
         args = (f'cloud undeploy --timeout {current_command.timeout}'
                 f' --connection-path "{common.elbus_path}" "{file}"')
         if config_var:
             for c in config_var:
                 args += f' --config-var "{c}"'
+        if config:
+            args += f' --config "{config}"'
         if test:
             args += ' --test'
         exec_cmd('eva-cloud-manager', args)
 
-    def update(self, repository_url, yes, info_only, test):
-        import json
-        from .tools import safe_download, set_file_lock
-        from .tools import remove_file_lock, get_arch
-        arch = get_arch()
-        if arch.startswith('arm'):
-            arch_sfx = 'armv7'
-        elif arch == 'x86_64':
-            arch_sfx = 'x86_64-musl'
-        elif arch == 'aarch64':
-            arch_sfx = 'aarch64-musl'
-        else:
-            raise RuntimeError(f'unsupported architecture: {arch}, '
-                               'please update the system manually')
-        try:
-            set_file_lock('update')
-        except RuntimeError:
-            raise RuntimeError('update is already active')
-        try:
-            os.environ['EVA_REPOSITORY_URL'] = repository_url
-            print('Using repository : ' +
-                  colored(repository_url, color='green', attrs=['bold']))
-            print()
-            info = get_node_svc_info()
-            build = info['build']
-            version = info['version']
-            if 'EVA_UPDATE_FORCE_VERSION' in os.environ:
-                new_version, new_build = os.environ[
-                    'EVA_UPDATE_FORCE_VERSION'].split(':')
-                new_build = int(new_build)
-                forced_version = True
-            else:
-                if test:
-                    update_info = 'update_info_test.json'
-                else:
-                    update_info = 'update_info.json'
-                result = json.loads(
-                    safe_download(f'{repository_url}/{update_info}'))
-                new_build = int(result['build'])
-                new_version = result['version']
-                forced_version = False
-            if info_only:
-                data = {
-                    'build': build,
-                    'new_build': new_build,
-                    'new_version': new_version,
-                    'update_available': build < new_build
-                }
-                print_result(data, need_header=False, name_value=True)
-                return
-            print(
-                colored('Current build', color='blue', attrs=['bold']) + ' : ' +
-                colored(f'{build} (v{version})', color='cyan'))
-            print(
-                colored(
-                    f'{"Selected" if forced_version else "Latest available"}'
-                    ' build',
-                    color='blue',
-                    attrs=['bold']) + ' : ' +
-                colored('{} (v{})'.format(new_build, new_version),
-                        color='yellow'))
-            if build == new_build:
-                return
-            if build > new_build:
-                raise RuntimeError(
-                    'Your build is newer than then update server has')
-            manifest = json.loads(
-                safe_download(f'{repository_url}/{new_version}/nightly/'
-                              f'manifest-{new_build}.json'))
-            if not yes:
-                if version != new_version:
-                    try:
-                        content = safe_download(
-                            f'{repository_url}/{new_version}/nightly/UPDATE.rst'
-                        )
-                    except Exception as e:
-                        err('Unable to download update info')
-                        raise
-                    print()
-                    print(content.decode())
-                try:
-                    u = input('Type ' +
-                              colored('YES', color='red', attrs=['bold']) +
-                              ' (uppercase) to update, or press ' +
-                              colored('ENTER', color='white') + ' to abort > ')
-                except:
-                    print()
-                    u = ''
-                if u != 'YES':
-                    return
-            update_script = f'update-{new_build}.sh'
-            update_files = [
-                update_script, f'eva-{new_version}-{new_build}-{arch_sfx}.tgz'
-            ]
-            print()
-            print('Updating...')
-            try:
-                os.chdir(common.dir_eva)
-                for f in update_files:
-                    url = f'{repository_url}/{new_version}/nightly/{f}'
-                    try:
-                        data = manifest['content'][f]
-                        sig = data['signature']
-                    except KeyError:
-                        raise RuntimeError(
-                            f'Signature for {url} not found in manifest')
-                    with open(f, 'wb') as fh:
-                        fh.write(
-                            safe_download(url,
-                                          manifest=manifest,
-                                          timeout=current_command.timeout * 20))
-                    try:
-                        if os.system(f'./sbin/rsatool verify "{f}" '
-                                     f'"./share/eva.key" "{sig}" > /dev/null'):
-                            raise RuntimeError(
-                                'Signature verification error for {url}')
-                    except:
-                        os.unlink(f)
-                        raise
-                if os.system(f'bash {update_script}'):
-                    err('FAILED TO APPLY UPDATE! '
-                        'TRYING TO BRING THE NODE BACK ONLINE')
-                    self.server_start()
-                    raise RuntimeError(f'update failed')
+    def cloud_update(self, nodes, check_timeout, all, info_only, yes):
+        if not nodes and not all:
+            raise RuntimeError('specify either nodes to update or --all')
+        args = (f'cloud update --timeout {current_command.timeout}'
+                f' --connection-path "{common.elbus_path}"'
+                f' --check-timeout "{check_timeout}"')
+        args += ' ' + ' '.join(nodes)
+        if all:
+            args += ' --all'
+        if info_only:
+            args += ' --info-only'
+        if yes:
+            args += ' --YES'
+        exec_cmd('eva-cloud-manager', args)
+
+    def update(self, download_timeout, repository_url, yes, info_only, test):
+        args = f'node update --timeout {current_command.timeout}'
+        if repository_url:
+            args = + f' --repository-url "{repository_url}"'
+        if download_timeout:
+            args += f' --download-timeout {download_timeout}'
+        if yes:
+            args += ' --YES'
+        if info_only:
+            args += ' --info-only'
+        if test:
+            args += ' --test'
+        old_ver = get_node_svc_info()
+        exec_cmd('eva-cloud-manager', args)
+        if not info_only:
+            if old_ver != get_node_svc_info():
                 print('Update completed', end='')
-            finally:
-                for f in update_files:
-                    try:
-                        os.unlink(f)
-                    except FileNotFoundError:
-                        pass
-            if common.interactive:
-                print('. Now exit EVA shell and log in back')
-            else:
-                print()
-            return
-        finally:
-            remove_file_lock('update')
+                if common.interactive:
+                    print('. Now exit EVA shell and log in back')
+                else:
+                    print()
 
     def registry_manage(self, yedb_args=None):
         cmd = f'{common.dir_eva}/venv/bin/yedb'
@@ -258,7 +169,8 @@ class CLI:
                  'the registry manager in offline mode')
             warn('The registry database will be '
                  'LOCKED until the registry shell exit')
-            if yedb_args is None or getch.getch() != '\x03':
+            warn('Press any key to continue, Ctrl-C to abort...')
+            if getch.getch() != '\x03':
                 os.system(f'env YEDB_PS=registry "{cmd}"'
                           f' "{common.dir_eva}/runtime/registry"{args}')
 
@@ -1237,20 +1149,57 @@ class CLI:
                 'name': 'CLI version',
                 'value': __version__
             }]
-        try:
-            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-            s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
-            s.connect(('255.255.255.255', 0))
-            ip = s.getsockname()[0]
-            try:
-                s.close()
-            except:
-                pass
-        except:
-            ip = None
+        ip = get_my_ip()
         if ip:
             if current_command.json:
                 d['ip'] = ip
             else:
                 d.append({'name': 'IP address', 'value': ip})
         print_result(d, need_header=False, name_value=False)
+
+    def mirror_set(self, mirror_url):
+        args = f'node mirror-set "{mirror_url}"'
+        exec_cmd('eva-cloud-manager', args)
+        self.venv_mirror_set(mirror_url=mirror_url)
+
+    def mirror_update(self, dest, current_arch_only, force, repository_url):
+        args = f'node mirror-update --timeout {current_command.timeout}'
+        if dest:
+            args += f' --dest "{dest}"'
+        if repository_url:
+            args = + f' --repository-url "{repository_url}"'
+        if force:
+            args += ' --force'
+        if current_arch_only:
+            args += ' --current-arch-only'
+        exec_cmd('eva-cloud-manager', args)
+        self.venv_mirror_update(dest=dest)
+        print()
+        self.mirror_info()
+
+    def mirror_info(self):
+        ip = get_my_ip()
+        port = 7780
+        if ip is None:
+            ip = '<host/ip>'
+        mirror_url = f'http://{ip}:{port}'
+        print(f'Mirror URL: ' +
+              colored(mirror_url, color='green', attrs=['bold']))
+        print()
+        print(f'EVA ICS update: -u http://{ip}:{port}/eva')
+        print(f'pip-extra-options: "-i {mirror_url}/pypi/local'
+              f' --trusted-host {ip}"')
+        print()
+        print('To automatically set the mirror config on'
+              ' other nodes, execute there a command:')
+        print()
+        print(
+            colored(f'    eva mirror set {mirror_url}',
+                    color='green',
+                    attrs=['bold']))
+        print()
+        print('Note: deploy the mirror service if not deployed.')
+        print('If the mirror is not created yet, '
+              'create it with "eva mirror update" command.')
+        print('If the mirror service is set to different port, '
+              'replace it in the URLs above.')
