@@ -1,4 +1,5 @@
 import sys
+import socket
 import os
 import time
 import neotermcolor
@@ -75,6 +76,21 @@ def remove_file_lock(name):
         pass
 
 
+def get_my_ip():
+    try:
+        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
+        s.connect(('255.255.255.255', 0))
+        ip = s.getsockname()[0]
+        try:
+            s.close()
+        except:
+            pass
+    except:
+        ip = None
+    return ip
+
+
 TIME_ORD = {
     'created': 0,
     'accepted': 1,
@@ -421,30 +437,3 @@ def get_node_svc_info():
 def get_arch():
     import platform
     return platform.machine()
-
-
-def safe_download(url, manifest=None, timeout=None, public_key=None, **kwargs):
-    import requests
-    r = requests.get(url,
-                     timeout=timeout if timeout else current_command.timeout,
-                     **kwargs)
-    if not r.ok:
-        raise RuntimeError(f'HTTP error {r.status_code} for {url}')
-    if isinstance(manifest, str):
-        r_m = requests.get(manifest, **kwargs)
-        if not r_m.ok:
-            raise RuntimeError(f'HTTP error {r.status_code} for {manifest}')
-        manifest = r_m.json()
-    if manifest:
-        try:
-            data = manifest['content'][url.rsplit('/', 1)[-1]]
-            size = data['size']
-            sha256 = data['sha256']
-        except KeyError:
-            raise RuntimeError(f'Info for {url} not found in manifest')
-        if len(r.content) != size:
-            raise RuntimeError(f'Size check error for {url}')
-        import hashlib
-        if hashlib.sha256(r.content).hexdigest() != sha256:
-            raise RuntimeError(f'Checksum error for {url}')
-    return r.content
